import com.grapecity.documents.excel.*;
import java.io.*;

public class Main {
    public static void main(String[] args) throws Exception {

        // the reason I only give 100,000*30 cells is that POI consumes nearly 4G memory in current benchmark, it will be out of memory
        int rowCount = 100000;
        int colCount = 30;

        String product = "";
        String scenario = "double";

        if (args.length != 0) {
            //product = args[0];
            scenario = args[0];
        }

        RefObject<Double> setTime = new RefObject<>();
        RefObject<Double> getTime = new RefObject<>();
        RefObject<Double> saveTime = new RefObject<>();
        RefObject<Double> usedMem = new RefObject<>();
        RefObject<Double> setTime2 = new RefObject<>();
        RefObject<Double> getTime2 = new RefObject<>();
        RefObject<Double> saveTime2 = new RefObject<>();
        RefObject<Double> usedMem2 = new RefObject<>();

        Workbook resultBook = new Workbook();
        IWorksheet summarySheet = resultBook.getWorksheets().get("Sheet1");
        summarySheet.setName("Summary");
        File benchmarks = new File("output/benchmarks.xlsx");
        if(benchmarks.exists()){
            resultBook.open("output/benchmarks.xlsx");
            summarySheet = resultBook.getWorksheets().get("Summary");
        }

        String scenarioLower = scenario.toLowerCase();
        if (scenarioLower.equals("double")) {
            IWorksheet resultSheet = createResultSheet(resultBook,"double");

            GcExcelBenchmark.TestSetRangeValues_Double(rowCount, colCount, setTime, getTime, saveTime, usedMem);
            FillResult(resultSheet, "gcexcel", setTime, getTime, saveTime, usedMem);


            POIBenchmark.TestSetRangeValues_Double(rowCount, colCount, setTime2, getTime2, saveTime2, usedMem2);
            FillResult(resultSheet, "poi", setTime2, getTime2, saveTime2, usedMem2);
            
            RefObject<Double> setDiff = new RefObject<Double>(setTime2.value - setTime.value);
            RefObject<Double> getDiff = new RefObject<Double>(getTime2.value - getTime.value);
            RefObject<Double> saveDiff = new RefObject<Double>(saveTime2.value - saveTime.value);
            RefObject<Double> memDiff = new RefObject<Double>(usedMem2.value - usedMem.value);
            RefObject<Double> setQuot = new RefObject<Double>(setTime2.value / setTime.value);
            RefObject<Double> getQuot = new RefObject<Double>(getTime2.value / getTime.value);
            RefObject<Double> saveQuot = new RefObject<Double>(saveTime2.value / saveTime.value);
            RefObject<Double> memQuot = new RefObject<Double>(usedMem2.value / usedMem.value);

            FillResult(resultSheet, "difference", setDiff, getDiff, saveDiff, memDiff);
            FillResult(resultSheet, "quotient", setQuot, getQuot, saveQuot, memQuot);
            resultSheet.getRange("B5:D5").setNumberFormat("0");

            com.grapecity.documents.excel.drawing.IShape shape = null;
            try {
                shape = summarySheet.getShapes().get("double_performance");
            }
            catch (Exception ex)
            {}
            if (shape == null)
            {   // create charts
                shape = summarySheet.getShapes().addChart(com.grapecity.documents.excel.drawing.ChartType.ColumnClustered, 10, 10, 300, 300);
                shape.getChart().getSeriesCollection().add(resultSheet.getRange("A1:C4"), com.grapecity.documents.excel.drawing.RowCol.Columns, true, true);
                shape.getChart().getSeriesCollection().get(0).setHasDataLabels(true);
                shape.getChart().getSeriesCollection().get(1).setHasDataLabels(true);
                shape.getChart().setHasTitle(true);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("Double Values Test - Performance");
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("GcExcel Java vs. Apache POI", 0);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().getColor().setRGB(Color.GetCornflowerBlue());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().setSize(15);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().getColor().setRGB(Color.GetOrange());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().setSize(10);
                shape.setName("double_performance");

                shape = summarySheet.getShapes().addChart(com.grapecity.documents.excel.drawing.ChartType.ColumnClustered, 320, 10, 300, 300);
                com.grapecity.documents.excel.drawing.ISeries series = shape.getChart().getSeriesCollection().newSeries();
                series.setFormula("=SERIES(double!B1,double!A5,double!B5,1");
                series.setHasDataLabels(true);
                series = shape.getChart().getSeriesCollection().newSeries();
                series.setFormula("=SERIES(double!C1,double!A5,double!C5,2");
                series.setHasDataLabels(true);
                shape.getChart().setHasTitle(true);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("Double Values Test - Memory");
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("GcExcel Java vs. Apache POI", 0);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().getColor().setRGB(Color.GetCornflowerBlue());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().setSize(15);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().getColor().setRGB(Color.GetOrange());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().setSize(10);
                shape.setName("double_memory");
            }
        } else if (scenarioLower.equals("string")) {
            IWorksheet resultSheet = createResultSheet(resultBook,"string");
            //resultSheet = createResultSheet(resultBook,"string");

            GcExcelBenchmark.TestSetRangeValues_String(rowCount, colCount, setTime, getTime, saveTime, usedMem);
            FillResult(resultSheet, "gcexcel", setTime, getTime, saveTime, usedMem);



            POIBenchmark.TestSetRangeValues_String(rowCount, colCount, setTime2, getTime2, saveTime2, usedMem2);
            FillResult(resultSheet, "poi", setTime2, getTime2, saveTime2, usedMem2);

            RefObject<Double> setDiff = new RefObject<Double>(setTime2.value - setTime.value);
            RefObject<Double> getDiff = new RefObject<Double>(getTime2.value - getTime.value);
            RefObject<Double> saveDiff = new RefObject<Double>(saveTime2.value - saveTime.value);
            RefObject<Double> memDiff = new RefObject<Double>(usedMem2.value - usedMem.value);
            RefObject<Double> setQuot = new RefObject<Double>(setTime2.value / setTime.value);
            RefObject<Double> getQuot = new RefObject<Double>(getTime2.value / getTime.value);
            RefObject<Double> saveQuot = new RefObject<Double>(saveTime2.value / saveTime.value);
            RefObject<Double> memQuot = new RefObject<Double>(usedMem2.value / usedMem.value);

            FillResult(resultSheet, "difference", setDiff, getDiff, saveDiff, memDiff);
            FillResult(resultSheet, "quotient", setQuot, getQuot, saveQuot, memQuot);
            resultSheet.getRange("B5:D5").setNumberFormat("0");

            com.grapecity.documents.excel.drawing.IShape shape = null;
            try {
                shape = summarySheet.getShapes().get("string_performance");
            }
            catch (Exception ex)
            {}
            if (shape == null)
            {   // create charts
                shape = summarySheet.getShapes().addChart(com.grapecity.documents.excel.drawing.ChartType.ColumnClustered, 10, 320, 300, 300);
                shape.getChart().getSeriesCollection().add(resultSheet.getRange("A1:C4"), com.grapecity.documents.excel.drawing.RowCol.Columns, true, true);
                shape.getChart().getSeriesCollection().get(0).setHasDataLabels(true);
                shape.getChart().getSeriesCollection().get(1).setHasDataLabels(true);
                shape.getChart().setHasTitle(true);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("String Values Test - Performance");
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("GcExcel Java vs. Apache POI", 0);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().getColor().setRGB(Color.GetCornflowerBlue());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().setSize(15);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().getColor().setRGB(Color.GetOrange());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().setSize(10);
                shape.setName("string_performance");

                shape = summarySheet.getShapes().addChart(com.grapecity.documents.excel.drawing.ChartType.ColumnClustered, 320, 320, 300, 300);
                com.grapecity.documents.excel.drawing.ISeries series = shape.getChart().getSeriesCollection().newSeries();
                series.setFormula("=SERIES(string!B1,string!A5,string!B5,1");
                series.setHasDataLabels(true);
                series = shape.getChart().getSeriesCollection().newSeries();
                series.setFormula("=SERIES(string!C1,string!A5,string!C5,2");
                series.setHasDataLabels(true);
                shape.getChart().setHasTitle(true);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("String Values Test - Memory");
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("GcExcel Java vs. Apache POI", 0);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().getColor().setRGB(Color.GetCornflowerBlue());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().setSize(15);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().getColor().setRGB(Color.GetOrange());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().setSize(10);
                shape.setName("string_memory");
            }
        } else if (scenarioLower.equals("date")) {
            IWorksheet resultSheet = createResultSheet(resultBook,"date");
            //resultSheet = createResultSheet(resultBook,"date");

            GcExcelBenchmark.TestSetRangeValues_Date(rowCount, colCount, setTime, getTime, saveTime, usedMem);
            FillResult(resultSheet, "gcexcel", setTime, getTime, saveTime, usedMem);


            POIBenchmark.TestSetRangeValues_Date(rowCount, colCount, setTime2, getTime2, saveTime2, usedMem2);
            FillResult(resultSheet, "poi", setTime2, getTime2, saveTime2, usedMem2);

            RefObject<Double> setDiff = new RefObject<Double>(setTime2.value - setTime.value);
            RefObject<Double> getDiff = new RefObject<Double>(getTime2.value - getTime.value);
            RefObject<Double> saveDiff = new RefObject<Double>(saveTime2.value - saveTime.value);
            RefObject<Double> memDiff = new RefObject<Double>(usedMem2.value - usedMem.value);
            RefObject<Double> setQuot = new RefObject<Double>(setTime2.value / setTime.value);
            RefObject<Double> getQuot = new RefObject<Double>(getTime2.value / getTime.value);
            RefObject<Double> saveQuot = new RefObject<Double>(saveTime2.value / saveTime.value);
            RefObject<Double> memQuot = new RefObject<Double>(usedMem2.value / usedMem.value);

            FillResult(resultSheet, "difference", setDiff, getDiff, saveDiff, memDiff);
            FillResult(resultSheet, "quotient", setQuot, getQuot, saveQuot, memQuot);
            resultSheet.getRange("B5:D5").setNumberFormat("0");

            com.grapecity.documents.excel.drawing.IShape shape = null;
            try {
                shape = summarySheet.getShapes().get("date_performance");
            }
            catch (Exception ex)
            {}
            if (shape == null)
            {   // create charts
                shape = summarySheet.getShapes().addChart(com.grapecity.documents.excel.drawing.ChartType.ColumnClustered, 10, 630, 300, 300);
                shape.getChart().getSeriesCollection().add(resultSheet.getRange("A1:C4"), com.grapecity.documents.excel.drawing.RowCol.Columns, true, true);
                shape.getChart().getSeriesCollection().get(0).setHasDataLabels(true);
                shape.getChart().getSeriesCollection().get(1).setHasDataLabels(true);
                shape.getChart().setHasTitle(true);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("Date Values Test - Performance");
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("GcExcel Java vs. Apache POI", 0);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().getColor().setRGB(Color.GetCornflowerBlue());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().setSize(15);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().getColor().setRGB(Color.GetOrange());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().setSize(10);
                shape.setName("date_performance");

                shape = summarySheet.getShapes().addChart(com.grapecity.documents.excel.drawing.ChartType.ColumnClustered, 320, 630, 300, 300);
                com.grapecity.documents.excel.drawing.ISeries series = shape.getChart().getSeriesCollection().newSeries();
                series.setFormula("=SERIES(date!B1,date!A5,date!B5,1");
                series.setHasDataLabels(true);
                series = shape.getChart().getSeriesCollection().newSeries();
                series.setFormula("=SERIES(date!C1,date!A5,date!C5,2");
                series.setHasDataLabels(true);
                shape.getChart().setHasTitle(true);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("Date Values Test - Memory");
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("GcExcel Java vs. Apache POI", 0);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().getColor().setRGB(Color.GetCornflowerBlue());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().setSize(15);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().getColor().setRGB(Color.GetOrange());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().setSize(10);
                shape.setName("date_memory");
            }
        } else if (scenarioLower.equals("bigfile")) {
            IWorksheet resultSheet = createResultSheet2(resultBook,"Large XLSX");
            //resultSheet = createResultSheet2(resultBook,"big_XLSX");

            RefObject<Double> openTime = new RefObject<>();
            RefObject<Double> calcTime = new RefObject<>();
            RefObject<Double> openTime2 = new RefObject<>();
            RefObject<Double> calcTime2 = new RefObject<>();

            GcExcelBenchmark.TestBigExcelFile(rowCount, colCount, openTime, calcTime, saveTime, usedMem);
            FillResult(resultSheet, "gcexcel", openTime, calcTime, saveTime, usedMem);



            POIBenchmark.TestBigExcelFile(rowCount, colCount, openTime2, calcTime2, saveTime2, usedMem2);
            FillResult(resultSheet, "poi", openTime2, calcTime2, saveTime2, usedMem2);
            
            RefObject<Double> openDiff = new RefObject<Double>(openTime2.value - openTime.value);
            RefObject<Double> calcDiff = new RefObject<Double>(calcTime2.value - calcTime.value);
            RefObject<Double> saveDiff = new RefObject<Double>(saveTime2.value - saveTime.value);
            RefObject<Double> memDiff = new RefObject<Double>(usedMem2.value - usedMem.value);
            RefObject<Double> openQuot = new RefObject<Double>(openTime2.value / openTime.value);
            RefObject<Double> calcQuot = new RefObject<Double>(calcTime2.value / calcTime.value);
            RefObject<Double> saveQuot = new RefObject<Double>(saveTime2.value / saveTime.value);
            RefObject<Double> memQuot = new RefObject<Double>(usedMem2.value / usedMem.value);

            FillResult(resultSheet, "difference", openDiff, calcDiff, saveDiff, memDiff);
            FillResult(resultSheet, "quotient", openQuot, calcQuot, saveQuot, memQuot);
            resultSheet.getRange("B5:D5").setNumberFormat("0");

            com.grapecity.documents.excel.drawing.IShape shape = null;
            try {
                shape = summarySheet.getShapes().get("bigfile_performance");
            }
            catch (Exception ex)
            {}
            if (shape == null)
            {   // create charts
                shape = summarySheet.getShapes().addChart(com.grapecity.documents.excel.drawing.ChartType.ColumnClustered, 10, 940, 300, 300);
                shape.getChart().getSeriesCollection().add(resultSheet.getRange("A1:C4"), com.grapecity.documents.excel.drawing.RowCol.Columns, true, true);
                shape.getChart().getSeriesCollection().get(0).setHasDataLabels(true);
                shape.getChart().getSeriesCollection().get(1).setHasDataLabels(true);
                shape.getChart().setHasTitle(true);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("Large XLSX Test - Performance");
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("GcExcel Java vs. Apache POI", 0);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().getColor().setRGB(Color.GetCornflowerBlue());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().setSize(15);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().getColor().setRGB(Color.GetOrange());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().setSize(10);
                shape.setName("bigfile_performance");

                shape = summarySheet.getShapes().addChart(com.grapecity.documents.excel.drawing.ChartType.ColumnClustered, 320, 940, 300, 300);
                com.grapecity.documents.excel.drawing.ISeries series = shape.getChart().getSeriesCollection().newSeries();
                series.setFormula("=SERIES('Large XLSX'!B1,'Large XLSX'!A5,'Large XLSX'!B5,1");
                series.setHasDataLabels(true);
                series = shape.getChart().getSeriesCollection().newSeries();
                series.setFormula("=SERIES('Large XLSX'!C1,'Large XLSX'!A5,'Large XLSX'!C5,2");
                series.setHasDataLabels(true);
                shape.getChart().setHasTitle(true);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("Large XLSX Test - Memory");
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("GcExcel Java vs. Apache POI", 0);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().getColor().setRGB(Color.GetCornflowerBlue());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().setSize(15);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().getColor().setRGB(Color.GetOrange());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().setSize(10);
                shape.setName("bigfile_memory");
            }
        } else if(scenario.equals("formula")){
            IWorksheet resultSheet = createResultSheet3(resultBook,"formulas");
            //resultSheet = createResultSheet3(resultBook,"formulas");

            RefObject<Double> calcTime = new RefObject<>();
            RefObject<Double> calcTime2 = new RefObject<>();


            rowCount = 20000;
            POIBenchmark.TestSetRangeFormulas(rowCount, colCount, setTime2, calcTime2, saveTime2, usedMem2);
            FillResult(resultSheet, "poi", setTime2, calcTime2, saveTime2, usedMem2);

            GcExcelBenchmark.TestSetRangeFormulas(rowCount, colCount, setTime, calcTime, saveTime, usedMem);
            FillResult(resultSheet, "gcexcel", setTime, calcTime, saveTime, usedMem);

            RefObject<Double> setDiff = new RefObject<Double>(setTime2.value - setTime.value);
            RefObject<Double> calcDiff = new RefObject<Double>(calcTime2.value - calcTime.value);
            RefObject<Double> saveDiff = new RefObject<Double>(saveTime2.value - saveTime.value);
            RefObject<Double> memDiff = new RefObject<Double>(usedMem2.value - usedMem.value);
            RefObject<Double> setQuot = new RefObject<Double>(setTime2.value / setTime.value);
            RefObject<Double> calcQuot = new RefObject<Double>(calcTime2.value / calcTime.value);
            RefObject<Double> saveQuot = new RefObject<Double>(saveTime2.value / saveTime.value);
            RefObject<Double> memQuot = new RefObject<Double>(usedMem2.value / usedMem.value);

            FillResult(resultSheet, "difference", setDiff, calcDiff, saveDiff, memDiff);
            FillResult(resultSheet, "quotient", setQuot, calcQuot, saveQuot, memQuot);
            resultSheet.getRange("B5:D5").setNumberFormat("0");


            com.grapecity.documents.excel.drawing.IShape shape = null;
            try {
                shape = summarySheet.getShapes().get("formula_performance");
            }
            catch (Exception ex)
            {}
            if (shape == null)
            {   // create charts
                shape = summarySheet.getShapes().addChart(com.grapecity.documents.excel.drawing.ChartType.ColumnClustered, 10, 1250, 300, 300);
                shape.getChart().getSeriesCollection().add(resultSheet.getRange("A1:C4"), com.grapecity.documents.excel.drawing.RowCol.Columns, true, true);
                shape.getChart().getSeriesCollection().get(0).setHasDataLabels(true);
                shape.getChart().getSeriesCollection().get(1).setHasDataLabels(true);
                shape.getChart().setHasTitle(true);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("Formulas Test - Performance");
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("GcExcel Java vs. Apache POI", 0);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().getColor().setRGB(Color.GetCornflowerBlue());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().setSize(15);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().getColor().setRGB(Color.GetOrange());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().setSize(10);
                shape.setName("formula_performance");

                shape = summarySheet.getShapes().addChart(com.grapecity.documents.excel.drawing.ChartType.ColumnClustered, 320, 1250, 300, 300);
                com.grapecity.documents.excel.drawing.ISeries series = shape.getChart().getSeriesCollection().newSeries();
                series.setFormula("=SERIES(formulas!B1,formulas!A5,formulas!B5,1");
                series.setHasDataLabels(true);
                series = shape.getChart().getSeriesCollection().newSeries();
                series.setFormula("=SERIES(formulas!C1,formulas!A5,formulas!C5,2");
                series.setHasDataLabels(true);
                shape.getChart().setHasTitle(true);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("Formulas Test - Memory");
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().add("GcExcel Java vs. Apache POI", 0);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().getColor().setRGB(Color.GetCornflowerBlue());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(0).getFont().setSize(15);
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().getColor().setRGB(Color.GetOrange());
                shape.getChart().getChartTitle().getTextFrame().getTextRange().getParagraphs().get(1).getFont().setSize(10);
                shape.setName("formula_memory");
            }
        }

        resultBook.save("output/benchmarks.xlsx");
    }

    private static IWorksheet createResultSheet(IWorkbook workbook, String name){
        IWorksheet resultSheet = workbook.getWorksheets().get(name);
        if(resultSheet == null){
            resultSheet = workbook.getWorksheets().add();
            resultSheet.setName(name);

            resultSheet.getRange("B1:E1").setValue(new Object[][]{
            {"GcExcel", "Apache POI", "Difference", "Quotient"}
            });
            resultSheet.getRange("A2:A5").setValue(new Object[][]{
                {"Set Values (s)"},
                {"Get Values (s)"},
                {"Save File (s)"},
                {"Memory (MB)"}
            });

            resultSheet.getNames().add("gcexcel", "B2:B5");
            resultSheet.getNames().add("poi", "C2:C5");
            resultSheet.getNames().add("difference", "D2:D5");
            resultSheet.getNames().add("quotient", "E2:E5");

        }


        return resultSheet;
    }

    private static IWorksheet createResultSheet2(IWorkbook workbook, String name){
        IWorksheet resultSheet = workbook.getWorksheets().get(name);
        if(resultSheet == null){
            resultSheet = workbook.getWorksheets().add();
            resultSheet.setName(name);

            resultSheet.getRange("B1:E1").setValue(new Object[][]{
            {"GcExcel", "Apache POI", "Difference", "Quotient"}
            });
            resultSheet.getRange("A2:A5").setValue(new Object[][]{
                {"Open File (s)"},
                {"Calculation (s)"},
                {"Save File (s)"},
                {"Memory (MB)"}
            });

            resultSheet.getNames().add("gcexcel", "B2:B5");
            resultSheet.getNames().add("poi", "C2:C5");
            resultSheet.getNames().add("difference", "D2:D5");
            resultSheet.getNames().add("quotient", "E2:E5");

        }


        return resultSheet;
    }

    private static IWorksheet createResultSheet3(IWorkbook workbook, String name){
        IWorksheet resultSheet = workbook.getWorksheets().get(name);
        if(resultSheet == null){
            resultSheet = workbook.getWorksheets().add();
            resultSheet.setName(name);

            resultSheet.getRange("B1:E1").setValue(new Object[][]{
            {"GcExcel", "Apache POI", "Difference", "Quotient"}
            });
            resultSheet.getRange("A2:A5").setValue(new Object[][]{
                {"Set Values (s)"},
                {"Calculation (s)"},
                {"Save File (s)"},
                {"Memory (MB)"}
            });

            resultSheet.getNames().add("gcexcel", "B2:B5");
            resultSheet.getNames().add("poi", "C2:C5");
            resultSheet.getNames().add("difference", "D2:D5");
            resultSheet.getNames().add("quotient", "E2:E5");

        }

        return resultSheet;
    }

    public static void FillResult(IWorksheet worksheet, String product,
                                                 RefObject<Double> setTime,
                                                 RefObject<Double> getTime,
                                                 RefObject<Double> saveTime,
                                                 RefObject<Double> usedMem) {

        String refersTo = worksheet.getNames().get(product).getRefersTo();
        String range = refersTo.substring(refersTo.indexOf("\"") + 1, refersTo.lastIndexOf("\""))  ;
        worksheet.getRange(range).setValue(new double[][]{
            {setTime.value},
            {getTime.value},
            {saveTime.value},
            {usedMem.value}
        });

    }

}
